@@ -50,6 +50,11 @@ from cloudrunner.util.shell import Console
 from cloudrunner.util.http import load_from_link
 from cloudrunner.util.http import parse_url
 
+try:
+    import readline
+except ImportError:
+    pass
+
 CONFIG = Config(CONFIG_SHELL_LOC)
 PLUGINS = {}
 LIBRARY_ITEM = re.compile(r'\[\w*\]\://')
@@ -77,7 +82,6 @@ class ShellRunner(object):
     """
 
     def __init__(self, *args, **kwargs):
-
         self._parser = _parser()
 
         try:
@@ -151,10 +155,30 @@ class Shell(cmd.Cmd):
         self.last_options = ""
         self.running = False
         self.lang = 'bash'
-
+        self.save_history = False
+        self.runtime_history = []
         self._api = None
+
         assert self.api
 
+    def preloop(self):
+        try:
+            histfile = os.environ.get("CLOUDRUNNER_HISTFILE")
+            if histfile:
+                readline.read_history_file(histfile)
+                self.save_history = True
+                histsize = os.environ.get("CLOUDRUNNER_HISTSIZE", 10000)
+                readline.set_history_length(histsize)
+        except Exception, ex:
+            print ex
+            pass
+
+    def _save_history(self):
+        if self.save_history:
+            histfile = os.environ.get("CLOUDRUNNER_HISTFILE")
+            if histfile:
+                readline.write_history_file(histfile)
+
     @property
     def nodes(self):
         if self.api.transport.mode == "server":
@@ -192,6 +216,7 @@ class Shell(cmd.Cmd):
         if line == "EOF":
             return self.do_quit(line)
         self.buffer.append(line)
+        self.runtime_history.append(line)
         if self.target == "local":
             return self.do_run("")
 
@@ -210,6 +235,10 @@ class Shell(cmd.Cmd):
         console.yellow("Choose from bash, sh, python, perl, "
                        "ruby, nodejs, puppet")
 
+    def do_history(self, arg):
+        console.yellow("Execution history:", bold=1)
+        console.white("\n".join(self.runtime_history))
+
     def do_lang(self, line):
         if not line:
             line = "bash"
@@ -234,6 +263,7 @@ class Shell(cmd.Cmd):
 
     def do_switch(self, target):
         """Switch CloudRunner target"""
+        self.runtime_history.append("#! switch %s" % target)
         if not target:
             self.target = "local"
         else:
@@ -245,6 +275,11 @@ class Shell(cmd.Cmd):
 
     def do_quit(self, arg):
         print
+        try:
+            if self.save_history:
+                self._save_history()
+        except Exception, ex:
+            pass
         self.api.transport.terminate(force=True)
         return True
 
@@ -396,14 +431,15 @@ class Shell(cmd.Cmd):
             # Nothing to run
             return
 
+        self._save_history()
+
         options = []
         if line.strip():
             _opts = [opt.strip() for opt in line.split()]
             for _opt in _opts:
                 k, _, v = _opt.partition("=")
                 if k and v:
-                    if k == "--resume":
-                        options.append('--resume=%s' % v)
+                    options.append(_opt)
         elif hasattr(self, "last_session_id"):
             options.append('--resume=%s' % self.last_session_id)
 
@@ -447,7 +483,7 @@ class Shell(cmd.Cmd):
 
                 if job_id != cur_job_id:
                     console.green("========== JOB: @%s %s ==========" % (
-                        msg.run_as, job_id[:8]))
+                        msg.run_as, job_id))
                     cur_job_id = job_id
 
                 if msg.node != last_node:
